/*  GSMP - General Sound Manipulation Program
 *  Copyright (C) 2000-2001 Valentin Ziegler and Ren Rebe
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/* Short Description:
 *   The executable main program.
 */

#include "App.hh"
#include "config.hh"

using namespace GSMP;
using namespace GSML;

int main (int argc, char *argv[] )
{
  cout << "Welcome to GSMP " << VERSION_STRING << endl;

  App* m_app;
  Manager m_manager;

  try
    {
      m_app = new App (argc, argv, string ("gsmp-main.so"), string ("Toolkit-gtk.plugin.so"), &m_manager);
      m_app -> Run ();
      cout << "Run returned" << endl;
      delete m_app;
    }

  /* We are friendly enough to catch some Exceptions here and do some
   * cryptic output, even so most users won't find this more informative
   * than a simple "Aborted.". Because of gcc bugs exception do not work
   * properly in multi-threaded programs ...
   */

  catch (Exception e)
    {
      cout << "Unhandled GSMP Exception:" << endl
	   << e << endl
	   << "Exiting." << endl;
      return -1;
    }

  catch (GSMX_utils::BaseException e)
    {
      cout << "Unhandled GSMP-utils Exception:" << endl
	   << e << endl
	   << "Exiting." << endl;
      return -1;
    }

  catch (GSML::BasicExcept e)
    {
      cout << "Unhandled GSML Exception:" << endl;
      cout << e.info << endl;
    }

  catch (...)
    {
      cout << "Unhandled Exception of unknown type" << endl
	   << "Exiting." << endl;
      return -1;
    }
  cout << "Application exited normally ;-)" << endl;
  return 0;
}
