/* Job.hh
 *   Copyright (C) 2001-2002 Valentin Ziegler and Ren Rebe
 *
 ***************************************************************************
 *                                                                         *
 *   This code is free software; you can redistribute it and / or modify   *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef THREAD_JOB_HH
#define THREAD_JOB_HH

// Those sigc++/thread guys are somewhat strange... Don't wanna have dummy arrays for pthread structs !
#define SIGC_THREAD_IMPL

#include "Thread.hh"
#include <queue>

#ifndef THREAD_MAX_JOB_CMDS
#define THREAD_MAX_JOB_CMDS 64
#endif

namespace GSMX_utils
{
  namespace Threads
  {
    using namespace std;

    namespace Private { class Command; class CMDWait;};

    class ExceptTerminated {};

    class Job : protected Thread
    {
    public:
      Job ();
      virtual ~Job ();
      
      bool IsRunning ();
      
    protected:
      friend class Funct__;
      friend class Command__;
      friend class Private::CMDWait;
      
      /* Master side */
      
      // Command Job to stop after current Loop () returned,
      // and all pending commands have been executed.
      // suspends caller until Jop has stopped.
      void SyncWait ();

      // Command Job to stop after current Loop () returned,
      // and all pending commands have been executed.
      // does not suspend caller.
      void AsyncWait ();
      
      // to be called by implementation's constructor
      void Ready (bool run_now);
 
      /* Master & Slave side */
      
      // Continue cyclic execution of Loop ()
      void Continue ();
      
      /* Slave side */
      
      // Stop cyclic execution of Loop () // MUST NOT be called from Master Side !!
      void Wait ();
      
      // Are there pending commands/function requests ?
      bool IsRequest ();
      
      // the thread's loop
      virtual void Loop () = 0;
      
    private:
      void Wait__ ();
      virtual void* main (void* data);
      void PushCmd (Private::Command* cmd);
      volatile bool waiting;
      volatile bool exit;
      volatile bool wait_after_loop;
      volatile bool start_waiting;
      queue <Private::Command*> commands; // queue for pending commands
      Mutex waiting__;                    // locks "waiting"
      Mutex on_request;                   // locked when slave is handling a Request
      Mutex commands__;                   // locks the command queue
      Mutex ctor;
      
      Condition new_request;
    };
    
    
  }; // end namespace Threads
}; // end namespace GSMX_utils

#endif // THREAD_JOB_HH
